#!/bin/perl -w
# ctd-all-filamentos.pl
#
# Traitement des fichiers SeaBird CTD acquis avec Seasave et traites avec Seasoft
# SBEProcessing. 
# Utilise les fichiers d'extensions .hdr et .asc 
# Traite les donnes de la CTD SBE19+ et SBE911+
# Genere un fichier au format ODV
#
# J Grelet IRD fevrier 2008  Filamentos - Lima - Peru
# 
# $I$ 

use strict "subs"; # necessite de declarer toutes les variables globales
#use diagnostics;

use Time::Local;
use Date::Manip;
use File::Basename;
#use Data::Dumper;
use Getopt::Long;
use Oceano::Seawater;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION       = '1.2';
our $AUTHOR        = 'jgrelet IRD feb 2008 FILAMENTOS cruise';
my  $DEBUG         = 0;
my  $ECHO;

# parametres de la campagne
my  $cycle_mesure = 'FILAMENTOS1';
my  $plateforme   = 'OLAYA';
my  $seasave_version  = '5.37';
my  $version;
my  $institut = 'IMARPE';
my  $type_19  = 'SBE19+';
my  $sn_19    = '19P47979-5148';
my  $type_911 = 'SBE911+';
my  $type;
my  $sn_911   = '09P17420-0502';
my  $pi       = 'GRADOS';

my  $CONTEXTE      = "";
my  $TIMEZONE      = "GMT";
my  $FORMAT_DATE   = "DMY";
my  $CODE          = "1A";
my  $date_debut    = "06/02/2008";
my  $date_fin      = "20/02/2008";
my  $cruisePrefix  = "fila1";

my  $help;
my  $odv           = 1;

my  $code          = -1;  # code pour l'entete

my  $hdr_file;
my  $odv_file;

my  $type_odv = "C" ;
my  $bottom_depth = 0;


my($station,$Pres,$T,$S,$T0,$T1,$Ox0v,$Ox0ml,$Ox0kg,$nbin,$S0,$S1,$C0,$FlC);
my($sigmateta,$sigmateta0,$sigmateta1,$sndvel);
my($lat, $lat_pos, $lat_deg, $lat_min, $lat_hemi);
my($long, $long_pos, $long_deg, $long_min, $long_hemi);
my($julien, $h_date, $date, $mois, $jour, $annee, $heure, $min, $sec, $time);
my $fileName;

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: ctd-filamentos.pl [options] <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug=[1-3]          debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "example:\n\$ perl ctd-all-filamentos-odv.pl --echo\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("debug=i"         => \$DEBUG,  
               "echo"            => \$ECHO,  	       
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi) = @_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
sub julian {
  my($jj,$h,$m,$s) = @_;
  my $dec = (($h * 3600) + ($m * 60) + $s ) / 86400;
  return( $jj + $dec ); 
}


#------------------------------------------------------------------------------
# entete ODV
#------------------------------------------------------------------------------
sub entete_odv { 
	#my $today = &dateFormat(undef,"%d/%m/%Y");
  my $today = '02/13/2008 18:27';
  print  ODV_FILE "//ODV Spreadsheet file : $odv_file\n"; 
  print  ODV_FILE "//Data treated : $today\n"; 
  print  ODV_FILE "//\n"; 
  print  ODV_FILE "Cruise\tStation\tType\tmon/day/yr\thh:mm\tLongitude [degrees_east]\tLatitude [degrees_north]\tBot. Depth [m]\tDepth [m]\tTEMP [C]\tPSAL [Psu]\tDENS [kg/m3]\tSVEL [m/s]\tDOX2 [micromole/kg]\tFLU2[Volts]\n"; 
}


#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&Date_Init( "TZ=UTC" );
#&usage if( $#ARGV == -1);
&get_options;

# variables locales
my $lines = 0;
my $Pmax = 0;

## initialise les donnees 
$Pres=$T=$S=$T0=$T1=$Ox0v=$Ox0ml=$Ox0kg=$nbin=$S0=$S1=$sigmateta0=$sigmateta1=$sndvel= 1e36;
$FlC=99999;

# lecture du premier fichier d'entete pour extraction des parametres
# generaux si ces derniers n'ont pas ete definit sur la ligne de commande.
my $first_file = 'sbe19+/data/fila1001.hdr';
open( FILE, $first_file );
#print STDERR "Lit : $first_file pour dcodage entete\n";
while( <FILE> ){
  if( /Ship\s*:\s*(.*)/) {
    if ( not defined $plateforme ) {
      ($plateforme) = $1;
       chomp $plateforme;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ ) {
    ($cycle_mesure) = $1 if ( not defined $cycle_mesure );
  }
  if( /Software Version\s+(\d+\.\d+)/ ) {
    ($seasave_version) = $1  if ( not defined $seasave_version );
  }
}  
close FILE;

# ouverture des fichiers de sortie, on met en minuscule, lower case
my ($name,$dir) =  fileparse $first_file;
$odv_file = lc $cycle_mesure.'_ctd_odv.txt' if (defined $odv);

if( $DEBUG ) {
  printf "%s\n%s\n%s\n%s\n%s\n",
          $institut,$plateforme,$type_911,$sn_911,$type_19,$sn_19,$pi;
  while( <DATA> ) {
	  open( FILE_HDL,$_ ) or die "$_ n'existe pas !\n";;
    print "Test : $_\n";
		close FILE_HDL;
  }
  print "\nExit ctd.pl ... debug mode\n";
  exit;
}

if (defined $odv) {
  open( ODV_FILE,   "+> $odv_file" )   or die "Can't open file : $odv_file\n";
  &entete_odv;
} 

# parcourt des fichiers .HDR et .asc
while( <DATA> ){
  open( FILE_HDL, $_ ) or die "$_ n'existe pas !\n";
  $fileName = $_;
	chomp($fileName);
  print STDERR "Lit : $fileName\t" if defined $ECHO;

  # les numeros de stations n'ont pas ete entres correctement dans les entetes
  # on utilise le numero du fichier correspondant
  if( $fileName =~ /.+$cruisePrefix(\d{3})/ ) {
    ($station) = ($1);
		#print STDERR  " st: $station "; 
  }  

  # on lit les fichiers d'extention .hdr contenant les entetes
  while( <FILE_HDL> ){    
		# decodage du type de sonde
    if( /Sea-Bird\s*(\w+)/ )	{
      $type = ($1 eq 'SBE19plus') ? $1 : 'SBE911plus'; 
    }
    # decodage de la position, date et heure entrees manuellement
    if( m[Date\s*:\s*(\d+)/(\d+)/(\d+)]) {
      ($jour,$mois,$annee) = ($1, $2, $3);	
    };	
    if( /Time\s*:\s*(\d+)[:hH](\d+)/ ) {  # a modifier suivant le contexte
      ($heure,$min)  = ($1,$2);
      # conversion en format US (MM/DD/YY) pour ParseDate
      $date = $mois."/".$jour."/".$annee." ".$heure.":".$min.":"."00";
      #print STDERR  "Date: $date\n";
      $date = &ParseDate( $date );
      # transforme le day_of_year en julian day
      $julien = &UnixDate($date,"%j") -1;
      $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M"),
                                  &UnixDate($date,"%S") );
      $h_date=&UnixDate($date,"%d/%m/%Y %H:%M:%S");
    }
    if( /Latitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /Longitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
    # decodage de la position, date et heure de l'entree NMEA (GPS)
    if( /System UpLoad Time =\s*(\w+)(\s+\d+\s+\d+\s+\d+:\d+:\d+)/)	{
			# pour Filamentos car seaterm met la date au format francais, on saute 
			# car deja rentre manuellement avec seaterm 
			if ( $1 eq 'Feb' ) {	
        ($time)  = "$1$2";	
        $date=&ParseDate($time);
        # transforme le day_of_year en julian day 0
        $julien = &UnixDate($time,"%j") -1;
        $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M"),
                                  &UnixDate($date,"%S") );
        $h_date = &UnixDate($date,"%d/%m/%Y %H:%M:%S");
      }
		}
    if( /NMEA Latitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /NMEA Longitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
  }
  
  close FILE_HDL;
  # on lit les fichiers d'extension .asc contenant les donnees
  # attention aux majuscules/minuscules !!!!!
  $fileName =~ s/\.HDR/\.asc/i;
  open( FILE_HDL, $fileName );
  while( <FILE_HDL> ){           
    # on expurge toutes les lignes des entetes ainsi que les lignes vides
    # n'est pas necessaire avec asciiout
    if( not ( /^[*#]/ || /^\s*$/) ) {
      # lit les donnees, eclate la ligne,utilise la ligne courante $_ par defaut
      ($Pres,$T0,$C0,$Ox0v,$FlC,$Ox0ml,$Ox0kg,$nbin,$S0,$sigmateta0,$sndvel) = split;
      next if( $Pres =~ /PrDM/i );
      $Pmax = $Pres if ($Pres > $Pmax);
      # utilise soit les capteurs primaires ou secondaires
      $T = $T0; $S = $S0; $sigmateta = $sigmateta0;
      # mets toutes les valeurs inferieur a Pres, a mettre en longopt
      if ($lines == 0) {
	      my $falseBin = 99999;      
        for( my $i = 0; $i <= $Pres; $i++ ) {
          $falseBin = $nbin if ( $i == $Pres ); 
	      }
      }
      else {
        #$T68 = $T0 * 1.00024; 
        #$sndvel=&sw_svel($S0,$T68,$Pres);
	      if (defined $odv) {
          printf ODV_FILE "%s\t%3d\t%s\t%s\t%8.4f\t%7.4f\t%6.1f\t%6.1f",
	        $cycle_mesure, $station, $type_odv, 
					#&dateFormat($date,"%m/%d/%Y\t%H:%M"), 
          &UnixDate($date,"%m/%d/%Y\t%H:%M"),
					$long_pos, $lat_pos, $bottom_depth, $Pres;
          printf ODV_FILE ($T > 1e35) ? "\t" : "\t%6.2f", $T;
          printf ODV_FILE ($S > 1e35) ? "\t" : "\t%6.2f", $S;
          printf ODV_FILE "\t%6.3f",   $sigmateta;
          printf ODV_FILE "\t%7.2f",   $sndvel;
          printf ODV_FILE "\t%7.3f",   $Ox0kg;
          printf ODV_FILE "\t%5.3f",   $FlC;
          printf ODV_FILE "\n";
	      }				
      }
      # re-initialise les donnees lues sauf $pres utilise pour le fichier d'entete
      $T=$S=$T0=$T1=$Ox0v=$Ox0ml=$Ox0kg=$nbin=$S0=$S1=$FlC= 1e+36;
      $sigmateta=$sigmateta0=$sigmateta1=$sndvel= 1e36;
      $lines++;
    }
  }
  printf STDERR "   %04d %s %02d%05.2f %s %03d%05.2f %s %5.4g %s\n",
	        $station, $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	        $long_hemi, $Pmax, $type if defined $ECHO;

	# init value before read next header station	
  $Pres =  $lat_min = $long_min = 1e36;
  $Pmax = $lat_deg = $long_deg = $station =  0;  
	$h_date = $lat_hemi = $long_hemi = '';
  $. = 0; # remet le compteur de ligne a zero
  $lines = 0;
  close FILE_HDL;
}


close ODV_FILE if defined $odv;


__DATA__
sbe19+/data/fila1001.hdr
sbe19+/data/fila1002.hdr
sbe19+/data/fila1003.hdr
sbe19+/data/fila1004.hdr
sbe19+/data/fila1005.hdr
sbe19+/data/fila1006.hdr
sbe19+/data/fila1007.hdr
sbe19+/data/fila1008.hdr
sbe911+/data/fila1009.hdr
sbe19+/data/fila1011.hdr
sbe19+/data/fila1012.hdr
sbe19+/data/fila1013.hdr
sbe19+/data/fila1014.hdr
sbe19+/data/fila1015.hdr
sbe19+/data/fila1016.hdr
sbe19+/data/fila1017.hdr
sbe19+/data/fila1018.hdr
sbe19+/data/fila1019.hdr
sbe19+/data/fila1020.hdr
sbe19+/data/fila1021.hdr
sbe19+/data/fila1022.hdr
sbe19+/data/fila1023.hdr
sbe911+/data/fila1024.hdr
sbe19+/data/fila1025.hdr
sbe19+/data/fila1026.hdr
sbe19+/data/fila1027.hdr
sbe911+/data/fila1028.hdr
sbe911+/data/fila1029.hdr
sbe911+/data/fila1030.hdr
sbe911+/data/fila1031.hdr
sbe19+/data/fila1032.hdr
sbe19+/data/fila1033.hdr
sbe911+/data/fila1034.hdr
sbe911+/data/fila1035.hdr
sbe911+/data/fila1036.hdr
sbe19+/data/fila1037.hdr
sbe19+/data/fila1038.hdr
sbe19+/data/fila1039.hdr
sbe19+/data/fila1040.hdr
sbe19+/data/fila1041.hdr
sbe19+/data/fila1042.hdr
sbe19+/data/fila1045.hdr
sbe911+/data/fila1046.hdr
sbe19+/data/fila1047.hdr
sbe911+/data/fila1048.hdr
sbe19+/data/fila1049.hdr
sbe911+/data/fila1050.hdr
sbe19+/data/fila1051.hdr
sbe911+/data/fila1052.hdr
sbe19+/data/fila1053.hdr
sbe911+/data/fila1054.hdr
sbe19+/data/fila1055.hdr
sbe19+/data/fila1056.hdr
sbe19+/data/fila1057.hdr
sbe19+/data/fila1058.hdr
sbe19+/data/fila1059.hdr
sbe19+/data/fila1060.hdr
sbe19+/data/fila1061.hdr
sbe19+/data/fila1062.hdr
sbe19+/data/fila1063.hdr
sbe19+/data/fila1064.hdr
sbe19+/data/fila1065.hdr
sbe19+/data/fila1066.hdr
sbe19+/data/fila1073.hdr
sbe911+/data/fila1074.hdr
sbe19+/data/fila1075.hdr
sbe911+/data/fila1076.hdr
sbe19+/data/fila1077.hdr
sbe911+//data/fila1078.hdr
sbe19+/data/fila1079.hdr
sbe911+//data/fila1080.hdr
sbe19+/data/fila1081.hdr
sbe19+/data/fila1082.hdr
sbe19+/data/fila1083.hdr
sbe19+/data/fila1084.hdr
sbe19+/data/fila1085.hdr
sbe19+/data/fila1086.hdr
sbe19+/data/fila1087.hdr
sbe19+/data/fila1088.hdr
sbe19+/data/fila1089.hdr
sbe19+/data/fila1090.hdr
sbe19+/data/fila1090.hdr
sbe19+/data/fila1091.hdr
sbe911+/data/fila1092.hdr
sbe911+/data/fila1099.hdr
sbe911+/data/fila1100.hdr
sbe911+/data/fila1102.hdr
sbe19+/data/fila1103.hdr
sbe911+/data/fila1104.hdr
sbe19+/data/fila1105.hdr
sbe911+/data/fila1106.hdr
sbe19+/data/fila1107.hdr
sbe19+/data/fila1108.hdr
sbe19+/data/fila1109.hdr
sbe19+/data/fila1110.hdr
sbe911+/data/fila1111.hdr
sbe911+/data/fila1112.hdr
sbe19+/data/fila1113.hdr
sbe19+/data/fila1114.hdr
sbe911+/data/fila1115.hdr
sbe19+/data/fila1116.hdr
sbe19+/data/fila1117.hdr
sbe19+/data/fila1118.hdr
sbe19+/data/fila1119.hdr
sbe911+/data/fila1120.hdr
sbe19+/data/fila1121.hdr
sbe911+/data/fila1122.hdr
sbe19+/data/fila1123.hdr
sbe911+/data/fila1124.hdr
sbe19+/data/fila1125.hdr
sbe911+/data/fila1126.hdr
sbe19+/data/fila1127.hdr
sbe911+/data/fila1128.hdr
sbe911+/data/fila1200.hdr
