#!/usr/bin/perl -w
#
# Traitement des fichiers SeaBird CTD acquis avec Seasave et traités avec Seasoft
# SBEProcessing. 
# Utilise les fichiers d'extensions .hdr et .asc 
# Genere un fichier au format ODV

use strict; # necessite de declarer toutes les variables globales
#use diagnostics;

use Time::Local;
use Date::Manip;
use File::Basename;
use Data::Dumper;
use Getopt::Long;
use Switch;
use Oceano::Seawater;
use Oceano::Convert;


#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION       = '1.2';
our $AUTHOR        = 'jgrelet IRD feb 2008 FILAMENTOS cruise';
my  $DEBUG         = 0;
my  $ECHO;

# surcharge par la valeurs saisies sur la ligne de commande
my  $version;
my  $institut;
my  $type;
my  $sn;
my  $pi;

# a mettre sous forme d'argument ligne de commande
my  $CONTEXTE      = "";
my  $TIMEZONE      = "GMT";
my  $FORMAT_DATE   = "DMY";
my  $CODE          = "1A";
my  $date_debut    = "06/02/2008";
my  $date_fin      = "20/02/2008";
my  $cruisePrefix  = "fila1";

my  $help;
my  $xml           = 1;
my  $ascii         = undef;
my  $dtd           = undef;
my  $odv           = undef;
my  $secondary     = undef;

my  $code          = -1;  # code pour l'entete

my  $hdr_file;
my  $ctd_file;
my  $odv_file;

my  $type_odv = "C" ;
my  $bottom_depth = 0;


my($cycle_mesure, $plateforme) = undef;
my($seasave_version)     = undef;
my($station,$Pres,$T,$S,$T0,$T1,$FlC,$Bat,$xmiss,$Par,$Spar,$Cpar,$Ox0v,$Ox0ml,$Ox0kg,$nbin,$S0,$S1,$C0);
my($sigmateta,$sigmateta0,$sigmateta1,$sndvel);
my($lat, $lat_pos, $lat_deg, $lat_min, $lat_hemi);
my($long, $long_pos, $long_deg, $long_min, $long_hemi);
my($julien, $h_date, $date, $mois, $jour, $annee, $heure, $min, $sec, $time);

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: ctd-filamentos-odv.pl [options] <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug=[1-3]          debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "    --cycle_mesure=<name>  cycle_mesure name\n";
  print STDERR   "    --plateforme=<name>    ship or plateforme name\n";
  print STDERR   "    --date_debut=JJ/MM/YYYY     starting date from cycle_mesure\n";
  print STDERR   "    --date_fin=JJ/MM/YYYY     end date from cycle_mesure\n";
  print STDERR   "    --institut=<name>      institute name\n";
  print STDERR   "    --code_oopc=<value>    processing code\n";
  print STDERR   "    --pi=<pi_name> \n";
  print STDERR   "    --ascii                ASCII output instead XML\n";
  print STDERR   "    --xml                  XML output (default)\n";
  print STDERR   "    --odv                  only ODV output\n";	
  print STDERR   "    --local                use local DTD\n";
  print STDERR   "    --secondary            use T/C secondary sensor\n";
  print STDERR   "    --sn=<serial_number>\n";
  print STDERR   "    --type=<instrument_type> \n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "example:\n\$ perl ctd-filamentos-odv.pl --cycle_mesure=FILAMENTOS1 --institut=IMARPE --plateforme='OYALA' --sn=09P17420-0502 --type=SBE911+ --pi='GRADOS' --date_debut=06/02/2008 --date_fin=20/02/2008 data/fila1???.hdr --echo --odv\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi) = @_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
sub julian {
  my($jj,$h,$m,$s) = @_;
  my $dec = (($h * 3600) + ($m * 60) + $s ) / 86400;
  return( $jj + $dec ); 
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("cycle_mesure=s"  => \$cycle_mesure, 
               "plateforme=s"    => \$plateforme,  
               "date_debut=s"    => \$date_debut,  
               "date_fin=s"      => \$date_fin,  
               "type=s"          => \$type,  
               "sn=s"            => \$sn,  
               "pi=s"            => \$pi,  
               "code_oopc=s"     => \$CODE,  	       
               "ascii"           => \$ascii,  
               "odv"             => \$odv,	
               "debug=i"         => \$DEBUG,  
               "echo"            => \$ECHO,  	       
               "institut=s"      => \$institut, 
               "local"           => \$dtd,  
               "secondary"       => \$secondary,  
               "version"         => \$version,  
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;
  $xml = undef if $ascii;    
}

#------------------------------------------------------------------------------
# entete ODV
#------------------------------------------------------------------------------
sub entete_odv { 
  my $today = &dateFormat(undef,"%d/%m/%Y");
  print  ODV_FILE "//ODV Spreadsheet file : $odv_file\n"; 
  print  ODV_FILE "//Data treated : $today\n"; 
  print  ODV_FILE "//\n"; 
  print  ODV_FILE "Cruise\tStation\tType\tmon/day/yr\thh:mm\ttLongitude [degrees_east]\tLatitude [degrees_north]\tBot. Depth [m]\tDepth [m]\tTEMP [C]\tPSAL [Psu]\tDENS [kg/m3]\tSVEL [m/s]\tDOX2 [micromole/kg]\tFLU2 [volts]\n"; 
}


#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&dateInit( "EN","GMT" );
&usage if( $#ARGV == -1);
&get_options;

# variables locales
my $lines = 0;
my $Pmax  = 0;

# initialise les donnees 
$Pres=$T=$S=$T0=$T1=$FlC=$Bat=$Par=$Spar=$Cpar=$Ox0v=$Ox0ml=$Ox0kg=$nbin=$S0=$S1=$sigmateta0=$sigmateta1=$sndvel= 1e36;

# lecture du premier fichier d'entete pour extraction des parametres
# generaux si ces derniers n'ont pas ete definit sur la ligne de commande.
open( DATA_FILE, $ARGV[0] );
while( <DATA_FILE> ){           # header contient l'entete
  if( /Ship\s*:\s*(.*)/) {
    if ( not defined $plateforme ) {
      ($plateforme) = $1;
       chomp $plateforme;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ ) {
    ($cycle_mesure) = $1 if ( not defined $cycle_mesure );
  }
  if( /Sea-Bird\s+(\w+\s+\d+)/ ) {
    ($type) = $1  if ( not defined $type );
  }
  if( /Software Version Seasave Win32 V\s+(\d+\.\d+)/ ) {
    ($seasave_version) = $1  if ( not defined $seasave_version );
  }
}  

# ouverture des fichiers de sortie, on met en minuscule, lower case
my $fileName =  $ARGV[0];
my ($name,$dir) =  fileparse $fileName;
$odv_file = lc $cycle_mesure.'_ctd_odv.txt' if (defined $odv);

print STDERR "\nUse secondary sensor\n" if (defined $secondary);

if( $DEBUG ) {
  printf "\nInput  file: %s\n", $hdr_file;
  printf "Output file: %s   mode: %s\n",$ctd_file, defined $xml ? 'XML' : 'ASCII'; 
  printf "%s\n%s\n%s\n%s\n%s\n",
          $institut,$plateforme,$type,$sn,$pi;
  for( my $i = 0; $i <= $#ARGV; $i++ ){
    print $ARGV[$i] . "\n";
  }
  print "\nExit ctd.pl ... debug mode\n";
  exit;
}

if (defined $odv) {
  open( ODV_FILE,   "+> $odv_file" )   or die "Can't open file : $odv_file\n";
  &entete_odv;
} 
close DATA_FILE;

# parcourt des fichiers .HDR et .asc
for( my $i = 0; $i <= $#ARGV; $i++ ){
  my $fileName = $ARGV[$i];
  open( DATA_FILE, $fileName );
  print STDERR  "Lit: $fileName" if defined $ECHO;

  # les numeros de stations n'ont pas ete entres correctement dans les entetes
  # on utilise le numero du fichier correspondant
  if( $fileName =~ /.+$cruisePrefix(\d{3})/ ) {
    $station = $1;
  }  

  # on lit les fichiers d'extention .hdr contenant les entetes
  while( <DATA_FILE> ){           
   
    #if( /Station\s*:\s*(\d+)/ )	{
    #  ($station)  = $1; 
    #}
    # decodage de la position, date et heure entrees manuellement
    if( m[Date\s*:\s*(\d+)/(\d+)/(\d+)]) {
      ($jour,$mois,$annee) = ($1, $2, $3);	
    };	
    if( /Heure\s*:\s*(\d+)[:hH](\d+):(\d+)/ ) {  # a modifier suivant le contexte
      ($heure,$min,$sec)  = ($1,$2,$3);
      # conversion en format US (MM/DD/YY) pour ParseDate
      $date = $mois."/".$jour."/".$annee." ".$heure.":".$min.":".$sec;
      #print STDERR  "Date: $date\n";
      $date = &ParseDate( $date );
      # transforme le day_of_year en julian day
      $julien = &date2julian($annee,$mois,$jour,$heure,$min,$sec);
      $h_date=&dateFormat($date,"%d/%m/%Y %h:%M:%S");
    }
    if( /Latitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /Longitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
    # decodage de la position, date et heure de l'entree NMEA (GPS)
    if( /System UpLoad Time =\s+(\.*)/)	{ # a modifier suivant le contexte
    ($time)  = /System UpLoad Time =\s+(\w+\s+\d+\s+\d+\s+\d+:\d+:\d+)/;	
			# formatte la date au format francais
      $h_date = &dateFormat($time, "%d/%m/%Y %X");
      # recupere le julian day 0
      $julien = &dateFormat($time, "%j");
    }
    if( /NMEA Latitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &positionDeci($lat_deg, $lat_min, $lat_hemi);
    }
    if( /NMEA Longitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &positionDeci($long_deg, $long_min, $long_hemi);
    }
  }
  close DATA_FILE;

  # on lit les fichiers d'extension .asc contenant les donnees
  # attention aux majuscules/minuscules !!!!!
  $fileName =~ s/\.HDR/\.asc/i;
  open( DATA_FILE, $fileName );
  while( <DATA_FILE> ){           
    # on expurge toutes les lignes des entetes ainsi que les lignes vides
    # n'est pas necessaire avec asciiout
    if( not ( /^[*#]/ || /^\s*$/) ) {
      # lit les donnees, eclate la ligne,utilise la ligne courante $_ par defaut
     ($Pres,$T0,$C0,$Ox0v,$FlC,$Ox0ml,$Ox0kg,$nbin,$S0,$sigmateta0,$sndvel) = split;
     next if( $Pres =~ /PrDM/i );
      $Pmax = $Pres if ($Pres > $Pmax);
      # utilise soit les capteurs primaires ou secondaires
      if (defined $secondary) {
        $T = $T1; $S = $S1; $sigmateta = $sigmateta1;
      } else {
        $T = $T0; $S = $S0; $sigmateta = $sigmateta0;
      }	      
      # mets toutes les valeurs inferieur a Pres, a mettre en longopt
      if ($lines == 0) {
	      my $falseBin = 99999;      
        for( my $i = 0; $i <= $Pres; $i++ ) {
          $falseBin = $nbin if ( $i == $Pres ); 
	      }
      }
      else {
        #$T68 = $T0 * 1.00024; 
        #$sndvel=&sw_svel($S0,$T68,$Pres);
	      if (defined $odv) {
          printf ODV_FILE "%s\t%3d\t%s\t%s\t%8.4f\t%7.4f\t%6.1f\t%6.1f",
	        $cycle_mesure, $station, $type_odv, 
					&dateFormat($date,"%m/%d/%Y\t%H:%M"), 
					$long_pos, $lat_pos, $bottom_depth, $Pres;
          printf ODV_FILE ($T > 1e35) ? "\t" : "\t%6.2f", $T;
          printf ODV_FILE ($S > 1e35) ? "\t" : "\t%6.2f", $S;
          printf ODV_FILE "\t%6.3f",   $sigmateta;
          printf ODV_FILE "\t%7.2f",   $sndvel;
          printf ODV_FILE "\t%7.3f",   $Ox0kg;
          printf ODV_FILE "\t%6.4f",   $FlC;
          printf ODV_FILE "\n";
	      }				
      }
      # re-initialise les donnees lues sauf $pres utilise pour le fichier d'entete
     $T=$S=$T0=$T1=$FlC=$Bat=$Par=$Spar=$Cpar=$Ox0v=$Ox0ml=$Ox0kg=$nbin=$S0=$S1= 1e36;
      $sigmateta=$sigmateta0=$sigmateta1=$sndvel= 1e36;
      $lines++;
    }
  }
  printf STDERR "   %03d %s %02d%05.2f %s %03d%05.2f %s %5.4g\n",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi, $Pmax if defined $ECHO;
  $Pres = 1e36;	  
  $Pmax = 0;
  $. = 0; # remet le compteur de ligne a zero
  $lines = 0;
  close DATA_FILE;
}
close ODV_FILE if defined $odv;

