#!c:\perl\bin\perl -w
#
# Traitement des fichiers SeaBird CTD acquis avec Seasave et traits avec Seasoft
# SBEProcessing. 
# Utilise les fichiers d'extensions .hdr et .asc 
# Les fichiers de sortie prennent le nom du cycle_mesure (entete) + extensions 
# et _ctd + extension eventuelle .xml
#
# J Grelet IRD fevrier 2008  Filamentos - Lima - Peru
# 
# $I$ 

use strict; # necessite de declarer toutes les variables globales
#use diagnostics;

use Time::Local;
use Date::Manip;
use File::Basename;
use Data::Dumper;
use Getopt::Long;
use Switch;
use Oceano::Seawater;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION       = '1.2';
our $AUTHOR        = 'jgrelet IRD feb 2008 FILAMENTOS cruise';
my  $DEBUG         = 0;
my  $ECHO;

# surcharge par la valeurs saisies sur la ligne de commande
my  $version;
my  $institut;
my  $type;
my  $sn;
my  $pi;

# a mettre sous forme d'argument ligne de commande
my  $CONTEXTE      = "";
my  $TIMEZONE      = "GMT";
my  $FORMAT_DATE   = "DMY";
my  $CODE          = "1A";
my  $date_debut    = "06/02/2008";
my  $date_fin      = "20/02/2008";
my  $cruisePrefix  = "fila1";

my  $help;
my  $xml           = 1;
my  $ascii         = undef;
my  $dtd           = undef;
my  $secondary     = undef;

my  $code          = -1;  # code pour l'entete

my  $hdr_file;
my  $ctd_file;

my($cycle_mesure, $plateforme) = undef;
my($seasave_version)     = undef;
my($station,$Pres,$T,$S,$T0,$T1,$FlC,$Bat,$xmiss,$Par,$Spar,$Cpar,$Ox0v,$Ox0ml,$Ox0kg,$nbin,$S0,$S1,$C0);
my($sigmateta,$sigmateta0,$sigmateta1,$sndvel);
my($lat, $lat_pos, $lat_deg, $lat_min, $lat_hemi);
my($long, $long_pos, $long_deg, $long_min, $long_hemi);
my($julien, $h_date, $date, $mois, $jour, $annee, $heure, $min, $sec, $time);

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: ctd-filamentos.pl [options] <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug=[1-3]          debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "    --cycle_mesure=<name>  cycle_mesure name\n";
  print STDERR   "    --plateforme=<name>    ship or plateforme name\n";
  print STDERR   "    --date_debut=JJ/MM/YYYY     starting date from cycle_mesure\n";
  print STDERR   "    --date_fin=JJ/MM/YYYY     end date from cycle_mesure\n";
  print STDERR   "    --institut=<name>      institute name\n";
  print STDERR   "    --code_oopc=<value>    processing code\n";
  print STDERR   "    --pi=<pi_name> \n";
  print STDERR   "    --ascii                ASCII output instead XML\n";
  print STDERR   "    --xml                  XML output (default)\n";
  print STDERR   "    --local                use local DTD\n";
  print STDERR   "    --secondary            use T/C secondary sensor\n";
  print STDERR   "    --sn=<serial_number>\n";
  print STDERR   "    --type=<instrument_type> \n";
  print STDERR   "\naccept short options like -d1 -t2\n\n";
  print STDERR   "example:\n\$ perl ctd-filamentos.pl --cycle_mesure=FILAMENTOS1 --institut=IMARPE --plateforme='OYALA' --sn=09P17420-0502 --type=SBE911+ --pi='GRADOS' --date_debut=06/02/2008 --date_fin=20/02/2008 data/fila1???.hdr --echo --local\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("cycle_mesure=s"  => \$cycle_mesure, 
               "plateforme=s"    => \$plateforme,  
               "date_debut=s"    => \$date_debut,  
               "date_fin=s"      => \$date_fin,  
               "type=s"          => \$type,  
               "sn=s"            => \$sn,  
               "pi=s"            => \$pi,  
               "code_oopc=s"     => \$CODE,  	       
               "ascii"           => \$ascii,  
               "debug=i"         => \$DEBUG,  
               "echo"            => \$ECHO,  	       
               "institut=s"     => \$institut, 
               "local"           => \$dtd,  
               "secondary"       => \$secondary,  
               "version"         => \$version,  
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;
  $xml = undef if $ascii;    
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi) = @_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
sub julian {
  my($jj,$h,$m,$s) = @_;
  my $dec = (($h * 3600) + ($m * 60) + $s ) / 86400;
  return( $jj + $dec ); 
}


#------------------------------------------------------------------------------
# voir documentation 
# http://www.seabird.com/pdf_documents/manuals/Seasave_5.37.pdf
#------------------------------------------------------------------------------
sub decode_con_file {
  my ($fileName) = @_;
  my ($p_cond_g,$p_cond_h,$p_cond_i,$p_cond_j,$p_cond_ctcor,$p_cond_cpcor);
  my ($p_cond_sn,$p_cond_date,$p_cond_slope,$p_cond_offset);
  my ($p_temp_f0,$p_temp_g,$p_temp_h,$p_temp_i,$p_temp_j);
  my ($p_temp_sn,$p_temp_date,$p_temp_slope,$p_temp_offset);
  my $sensor;

  $sensor = (defined $secondary) ? 'SECONDARY' : 'PRIMARY';

  # attention, verifier si les fichiers sont en majuscules ou minuscules !!!!
  $fileName =~ s/\.HDR/\.CON/i;
  open( DATA_CON, $fileName );
  while( <DATA_CON> ){           
    chomp;
    # chomp ne marche pas correctement car le fichier .con est au format DOS
    # avec un CRLF (^M^J ou OD OA) a la fin de chaque ligne
    # on vire le CR si present en fin de ligne
    $_ =~ s/^(.+)\r$/$1/;
    switch( $. ) {
       if (defined $secondary)
       {       
         # conductivity	secondary sensor    
         case 6   { $p_cond_sn =  $_; }
         case 8   { (undef,undef,$p_cond_slope,$p_cond_offset) = split; }
         case 112 { ($p_cond_g,$p_cond_h,$p_cond_i,$p_cond_j,$p_cond_ctcor,
                     $p_cond_cpcor) = split; }
         case 54  { $p_cond_date =  $_; }
         # temperature secondary sensor 
         case 9   { $p_temp_sn =  $_; }
         case 5   { (undef,undef,undef,undef,undef,$p_temp_slope,
                     $p_temp_offset) = split; }
         case 113 { ($p_temp_f0,$p_temp_g,$p_temp_h,$p_temp_i,$p_temp_j) = split; }
         case 55  { $p_temp_date =  $_; }
       } 
       else {
         # conductivity	primary sensor     
         case 1   { $p_cond_sn =  $_; }
         case 3   { (undef,undef,$p_cond_slope,$p_cond_offset) = split; }
         case 110 { ($p_cond_g,$p_cond_h,$p_cond_i,$p_cond_j,$p_cond_ctcor,
                     $p_cond_cpcor) = split; }
         case 52  { $p_cond_date =  $_; }
         # temperature primary sensor
         case 4   { $p_temp_sn =  $_; }
         case 5   { (undef,undef,undef,undef,undef,$p_temp_slope,
                     $p_temp_offset) = split; }
         case 111 { ($p_temp_f0,$p_temp_g,$p_temp_h,$p_temp_i,$p_temp_j) = split; }
         case 53  { $p_temp_date =  $_; }
       }	       
    }  
  }
  close DATA_CON;

  #my $xml = qq|      
  #        <ETALONNAGE DATE=\"$p_cond_date\" CAPTEUR=\"COND\" NUMERO_SERIE=\"$p_cond_sn\" NUMERO=\"PRIMARY\">  
  #        <PARAMETRE VALEUR=\"G\">$p_cond_g</PARAMETRE>
  #|;
  #print CTD_FILE $xml;
  print CTD_FILE "    <INSTRUMENT TYPE=\"SBE04\" NUMERO_SERIE=\"$p_cond_sn\" CODE_ROSCOP=\"PSAL\" PERE=\"$sn\" NUMERO=\"$sensor\">\n"; 
  print CTD_FILE "      <PARAMETRE CLE=\"DATE\" TYPE=\"D\">$p_cond_date</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"H\" TYPE=\"N\">$p_cond_h</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"I\" TYPE=\"N\">$p_cond_i</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"J\" TYPE=\"N\">$p_cond_j</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"CTCOR\" TYPE=\"N\">$p_cond_ctcor</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"CPCOR\" TYPE=\"N\">$p_cond_cpcor</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"SLOPE\" TYPE=\"N\">$p_cond_slope</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"OFFSET\" TYPE=\"N\">$p_cond_offset</PARAMETRE>\n";
  print CTD_FILE "    </INSTRUMENT>\n";  
  print CTD_FILE "    <INSTRUMENT TYPE=\"SBE03\" NUMERO_SERIE=\"$p_temp_sn\" CODE_ROSCOP=\"TEMP\" PERE=\"$sn\" NUMERO=\"$sensor\">\n"; 
  print CTD_FILE "      <PARAMETRE CLE=\"DATE\" TYPE=\"D\">$p_temp_date</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"G\" TYPE=\"N\">$p_temp_g</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"H\" TYPE=\"N\">$p_temp_h</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"I\" TYPE=\"N\">$p_temp_i</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"J\" TYPE=\"N\">$p_temp_j</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"F0\" TYPE=\"N\">$p_temp_f0</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"SLOPE\" TYPE=\"N\">$p_temp_slope</PARAMETRE>\n";
  print CTD_FILE "      <PARAMETRE CLE=\"OFFSET\" TYPE=\"N\">$p_temp_offset</PARAMETRE>\n";
  print CTD_FILE "    </INSTRUMENT>\n";  
}

#------------------------------------------------------------------------------
# entete XML
#------------------------------------------------------------------------------
sub entete_xml { 
  my($fileName) = @_;
  my $today = &UnixDate( &ParseDate("today"), "%d/%m/%Y");
  
  print  CTD_FILE '<?xml version="1.0" encoding="ISO-8859-1"?>' . "\n"; 
  # les commentaires ne sont pas accepts par XML Toolbox Matlab de Geodise
  if ( defined $dtd ) {
		#print  CTD_FILE '<!DOCTYPE OCEANO SYSTEM "M:\ACQUISIT\oceano.dtd">' . "\n";
		print  CTD_FILE '<!DOCTYPE OCEANO SYSTEM "/d/CAMPAGNES/oceano.dtd">' . "\n";
		#print  CTD_FILE '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://antea/acquisit/oceano.dtd">' . "\n";
  } else {  
    print  CTD_FILE '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://www.brest.ird.fr/us191/database/oceano.dtd">' . "\n";
  }
  print  CTD_FILE '<OCEANO TYPE="PROFIL">' . "\n";
  print  CTD_FILE "  <ENTETE>\n";
  print  CTD_FILE "    <PLATEFORME>\n";
  print  CTD_FILE "      <LIBELLE>$plateforme</LIBELLE>\n";  
  print  CTD_FILE "    </PLATEFORME>\n";
  print  CTD_FILE "    <CYCLE_MESURE CONTEXTE=\"$CONTEXTE\" TIMEZONE=\"$TIMEZONE\" FORMAT=\"$FORMAT_DATE\">\n";  
  print  CTD_FILE "      <LIBELLE>$cycle_mesure</LIBELLE>\n";  
  print  CTD_FILE "      <DATE_DEBUT>$date_debut</DATE_DEBUT>\n";  
  print  CTD_FILE "      <DATE_FIN>$date_fin</DATE_FIN>\n";  
  print  CTD_FILE "      <INSTITUT>$institut</INSTITUT>\n";  
  print  CTD_FILE "      <RESPONSABLE>$pi</RESPONSABLE>\n"; 
  print  CTD_FILE "      <ACQUISITION LOGICIEL=\"SEASAVE\" VERSION=\"$seasave_version\"></ACQUISITION>\n"; 
  print  CTD_FILE "      <TRAITEMENT LOGICIEL=\"SBEDATAPROCESSING\" VERSION=\"$seasave_version\"></TRAITEMENT>\n"; 
  print  CTD_FILE "      <VALIDATION LOGICIEL=\"$0\" VERSION=\"$VERSION\" DATE=\"$today\" OPERATEUR=\"$pi\" CODIFICATION=\"OOPC\">\n";
  print  CTD_FILE "        <CODE>$CODE</CODE>\n";	    
  print  CTD_FILE "        <COMMENTAIRE>Extraction ralise avant la post-calibration</COMMENTAIRE>\n";
  print  CTD_FILE "      </VALIDATION>\n";  
  print  CTD_FILE "    </CYCLE_MESURE>\n";  
  print  CTD_FILE "    <INSTRUMENT TYPE=\"$type\" NUMERO_SERIE=\"$sn\">\n"; 
  print  CTD_FILE "    </INSTRUMENT>\n";  
  
  &decode_con_file($fileName);
  
  print  CTD_FILE "  </ENTETE>\n";  
  print  CTD_FILE "  <DATA>\n";  
}

#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&Date_Init( "TZ=UTC" );
&usage if( $#ARGV == -1);
&get_options;

# variables locales
my $lines = 0;
my $Pmax = 0;

# initialise les donnees 
$Pres=$T=$S=$T0=$T1=$FlC=$Bat=$Par=$Spar=$Cpar=$Ox0v=$Ox0ml=$Ox0kg=$nbin=$S0=$S1=$sigmateta0=$sigmateta1=$sndvel= 1e36;

# lecture du premier fichier d'entete pour extraction des parametres
# generaux si ces derniers n'ont pas ete definit sur la ligne de commande.
open( DATA_FILE, $ARGV[0] );
while( <DATA_FILE> ){           # header contient l'entete
  if( /Ship\s*:\s*(.*)/) {
    if ( not defined $plateforme ) {
      ($plateforme) = $1;
       chomp $plateforme;  # enleve le dernier caractere \n car motif (.*)
    }  
  }
  if( /Cruise\s*:\s*(\S+)/ ) {
    ($cycle_mesure) = $1 if ( not defined $cycle_mesure );
  }
  if( /Sea-Bird\s+(\w+\s+\d+)/ ) {
    ($type) = $1  if ( not defined $type );
  }
  if( /Software Version Seasave V\s+(\d+\.\d+)/ ) {
    ($seasave_version) = $1  if ( not defined $seasave_version );
  }
}  

# ouverture des fichiers de sortie, on met en minuscule, lower case
my $fileName =  $ARGV[0];
my ($name,$dir) =  fileparse $fileName;
$ctd_file = lc $cycle_mesure.'_ctd';
$ctd_file .= '.xml' if defined $xml;
$hdr_file = lc  $cycle_mesure.'.ctd';

print STDERR "\nUse secondary sensor\n" if (defined $secondary);

if( $DEBUG ) {
  printf "\nInput  file: %s\n", $hdr_file;
  printf "Output file: %s   mode: %s\n",$ctd_file, defined $xml ? 'XML' : 'ASCII'; 
  printf "%s\n%s\n%s\n%s\n%s\n",
          $institut,$plateforme,$type,$sn,$pi;
  for( my $i = 0; $i <= $#ARGV; $i++ ){
    print $ARGV[$i] . "\n";
  }
  print "\nExit ctd.pl ... debug mode\n";
  exit;
}

open( CTD_FILE, "+> $ctd_file" ) or die "Can't open file : $ctd_file\n";
open( HDR_FILE, "+> $hdr_file" ) or die "Can't open file : $hdr_file\n";

# ecriture des entetes
if ( defined $xml ) {
  &entete_xml( $fileName );
} else {	
  print CTD_FILE "$cycle_mesure  $plateforme  $institut  $type  $sn  $pi\n";
}

# Les paramtres supplmentaires
#DOX2;DISSOLVED OXYGEN;micromole/kg;0;450;%7.3lf;
#FLU2;FLUORESCENCE;milligram/m3;0;1;%8.4lf;
#TUR2;LIGHT ATTENUATION COEFFICIENT;m-1;0;10;%6.3lf;
#LGHT;LIGHT IRRADIANCE IMMERGED PAR;micromole photon/(m2.s);0;4000;%8.3lf;
#LGH4;LIGHT IRRADIANCE SURFACE PAR;micromole photon/(m2.s);0;3000;%8.3lf;
#LGH5;IMMERGED/SURF IRRADIANCE RATIO;%;0;100;%6.2lf;

print CTD_FILE "PRFL  PRES  TEMP    PSAL    DENS     SVEL     DOX2    FLU2    NAVG\n";
print HDR_FILE "$cycle_mesure  $plateforme  $institut  $type  $sn  $pi\n";
print HDR_FILE "St    Date      Heure    Latitude   Longitude  Profondeur\n\n";
close DATA_FILE;

# parcourt des fichiers .HDR et .asc
for( my $i = 0; $i <= $#ARGV; $i++ ){
  my $fileName = $ARGV[$i];
  open( DATA_FILE, $fileName );
  print STDERR  "Lit: $fileName" if defined $ECHO;

	# les numeros de stations n'ont pas ete entres correctement dans les entetes
  # on utilise le numero du fichier correspondant
  if( $fileName =~ /.+$cruisePrefix(\d{3})/ ) {
    ($station) = ($1);
  }  

  # on lit les fichiers d'extention .hdr contenant les entetes
  while( <DATA_FILE> ){           
    
    #if( /Station\s*:\s*(\d+)/ )	{
    #  ($station)  = ($1); 
    #}
    # decodage de la position, date et heure entrees manuellement
    if( m[Date\s*:\s*(\d+)/(\d+)/(\d+)]) {
      ($jour,$mois,$annee) = ($1, $2, $3);	
    };	
    if( /Heure\s*:\s*(\d+)[:hH](\d+):(\d+)/ ) {  # a modifier suivant le contexte
      ($heure,$min,$sec)  = ($1,$2,$3);
      # conversion en format US (MM/DD/YY) pour ParseDate
      $date = $mois."/".$jour."/".$annee." ".$heure.":".$min.":".$sec;
      #print STDERR  "Date: $date\n";
      $date = &ParseDate( $date );
      # transforme le day_of_year en julian day
      $julien = &UnixDate($time,"%j") -1;
      $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M"),
                                  &UnixDate($date,"%S") );
      $h_date=&UnixDate($date,"%d/%m/%Y %H:%M:%S");
    }
    if( /Latitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /Longitude\s*:\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
    # decodage de la position, date et heure de l'entree NMEA (GPS)
    if( /System UpLoad Time =\s+(\.*)/)	{ # a modifier suivant le contexte
    ($time)  = /System UpLoad Time =\s+(\w+\s+\d+\s+\d+\s+\d+:\d+:\d+)/;	
    $date=&ParseDate($time);
    # transforme le day_of_year en julian day 0
    $julien = &UnixDate($time,"%j") -1;
    $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M"),
                                &UnixDate($date,"%S") );
    $h_date = &UnixDate($date,"%d/%m/%Y %H:%M:%S");
    }
    if( /NMEA Latitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /NMEA Longitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
    }
  }
  
  # ecriture de l'entete (pression = -1, avec 5 decimales
  printf CTD_FILE "%3d  %4d %9.5f %8.5f %9.5f %s 1e+36 1e+36 1e+36\n",$station,
         $code,$julien,$lat_pos, $long_pos,&UnixDate($date,"%q");
  close DATA_FILE;
  # on lit les fichiers d'extension .asc contenant les donnees
  # attention aux majuscules/minuscules !!!!!
  $fileName =~ s/\.HDR/\.asc/i;
  open( DATA_FILE, $fileName );
  while( <DATA_FILE> ){           
    # on expurge toutes les lignes des entetes ainsi que les lignes vides
    # n'est pas necessaire avec asciiout
    if( not ( /^[*#]/ || /^\s*$/) ) {
      # lit les donnees, eclate la ligne,utilise la ligne courante $_ par defaut
      ($Pres,$T0,$C0,$Ox0v,$FlC,$Ox0ml,$Ox0kg,$nbin,$S0,$sigmateta0,$sndvel) = split;
      next if( $Pres =~ /PrDM/i );
      $Pmax = $Pres if ($Pres > $Pmax);
      # utilise soit les capteurs primaires ou secondaires
      if (defined $secondary) {
        $T = $T1; $S = $S1; $sigmateta = $sigmateta1;
      } else {
        $T = $T0; $S = $S0; $sigmateta = $sigmateta0;
      }	      
      # mets toutes les valeurs inferieur a Pres, a mettre en longopt
      if ($lines == 0) {
	      my $falseBin = 99999;      
	      for( my $i = 0; $i <= $Pres; $i++ ) {
	        $falseBin = $nbin if ( $i == $Pres ); 
          printf CTD_FILE "%3d  %4d  %6.5g  %6.5g  %6.3f  %7.2f  %7.6g %7.3g %3d\n", 
					                 $station,$i,$T,$S,$sigmateta,$sndvel,$Ox0kg,$FlC,$falseBin;
	}
      }
      else {
        #$T68 = $T0 * 1.00024; 
        #$sndvel=&sw_svel($S0,$T68,$Pres);
        printf CTD_FILE "%3d  %4d  %6.5g  %6.5g  %6.3f  %7.2f  %7.6g %7.3g %3d\n", 
				                 $station,$Pres,$T,$S,$sigmateta,$sndvel,$Ox0kg,$FlC,$nbin;
      }
      # re-initialise les donnees lues sauf $pres utilise pour le fichier d'entete
      $T=$S=$T0=$T1=$FlC=$Bat=$Par=$Spar=$Cpar=$Ox0v=$Ox0ml=$Ox0kg=$nbin=$S0=$S1= 1e36;
      $sigmateta=$sigmateta0=$sigmateta1=$sndvel= 1e36;
      $lines++;
    }
  }
  printf HDR_FILE  "%04d %s %02d%05.2f %s %03d%05.2f %s %4.4g\n",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi, $Pmax;
  printf STDERR "   %04d %s %02d%05.2f %s %03d%05.2f %s %5.4g\n",$station,
          $h_date, $lat_deg, $lat_min, $lat_hemi, $long_deg, $long_min, 
	  $long_hemi, $Pmax if defined $ECHO;
  $Pres = 1e36;
  $Pmax = 0;  
  $. = 0; # remet le compteur de ligne a zero
  $lines = 0;
  close DATA_FILE;
}

# return result to matlab
print STDERR $ctd_file;

close HDR_FILE;
if( defined $xml ) {
  print  CTD_FILE "  </DATA>\n";  
  print  CTD_FILE "</OCEANO>\n";  
}  
close CTD_FILE;

